// package.json
var package_default = {
  name: "create-wagmi",
  description: "Create wagmi apps with one command",
  version: "0.1.13",
  type: "module",
  license: "MIT",
  bin: {
    "create-wagmi": "dist/index.js"
  },
  scripts: {
    build: "tsup",
    "changeset:release": "pnpm build && changeset publish",
    "changeset:version": "changeset version && pnpm install --lockfile-only",
    dev: "tsup --watch",
    lint: "eslint . --cache",
    "lint:fix": "pnpm lint --fix",
    "lint:format": "prettier --write",
    prepare: "npx simple-git-hooks",
    typecheck: "tsc --noEmit"
  },
  dependencies: {
    cac: "^6.7.14",
    cpy: "^9.0.1",
    "cross-spawn": "^7.0.3",
    "detect-package-manager": "^2.0.1",
    execa: "^6.1.0",
    "fs-extra": "^10.1.0",
    ora: "^6.1.2",
    picocolors: "^1.0.0",
    prompts: "^2.4.2",
    "update-check": "^1.5.4",
    "validate-npm-package-name": "^4.0.0"
  },
  devDependencies: {
    "@changesets/changelog-github": "^0.4.4",
    "@changesets/cli": "^2.22.0",
    "@types/cross-spawn": "^6.0.2",
    "@types/eslint": "^8.4.6",
    "@types/fs-extra": "^9.0.13",
    "@types/node": "^18.11.7",
    "@types/prompts": "^2.4.1",
    "@types/validate-npm-package-name": "^4.0.0",
    "@typescript-eslint/eslint-plugin": "^5.37.0",
    "@typescript-eslint/parser": "^5.37.0",
    eslint: "8.17.0",
    "eslint-config-prettier": "^8.5.0",
    "eslint-import-resolver-typescript": "^2.7.1",
    "eslint-plugin-eslint-comments": "^3.2.0",
    "eslint-plugin-import": "^2.26.0",
    "eslint-plugin-prettier": "^4.2.1",
    "eslint-plugin-react-hooks": "^4.6.0",
    "lint-staged": "^13.0.3",
    prettier: "^2.7.1",
    "simple-git-hooks": "^2.8.1",
    tsup: "^6.2.3",
    typescript: "^4.9.4"
  },
  "simple-git-hooks": {
    "pre-commit": "pnpm lint-staged"
  },
  "lint-staged": {
    "*.{js,ts,tsx}": [
      "pnpm lint:fix"
    ],
    "*.{json,md,yml}": [
      "pnpm lint:format"
    ]
  },
  files: [
    "dist",
    "templates"
  ],
  repository: {
    type: "git",
    url: "https://github.com/wagmi-dev/create-wagmi.git"
  }
};

// src/utils/createTemplate.ts
function createTemplate(template) {
  return template;
}

// src/utils/getPackageManager.ts
import { detect } from "detect-package-manager";
var getPackageManager = ({ options }) => {
  if (options) {
    if (options.npm)
      return "npm";
    if (options.pnpm)
      return "pnpm";
    if (options.yarn)
      return "yarn";
  }
  const userAgent = process.env.npm_config_user_agent;
  if (userAgent) {
    if (userAgent.includes("pnpm"))
      return "pnpm";
    if (userAgent.includes("npm"))
      return "npm";
    if (userAgent.includes("yarn"))
      return "yarn";
  }
  return detect();
};

// src/utils/notifyUpdate.ts
import pico from "picocolors";
import checkForUpdate from "update-check";
var log = console.log;
async function notifyUpdate({ options }) {
  try {
    const res = await checkForUpdate(package_default);
    if (res == null ? void 0 : res.latest) {
      const packageManager = await getPackageManager({ options });
      const updateMessage = packageManager === "pnpm" ? "pnpm add -g create-wagmi" : packageManager === "yarn" ? "yarn global add create-wagmi" : "npm i -g create-wagmi";
      log(
        pico.bold(
          pico.yellow("A new version of `create-wagmi` is available!") + "\nYou can update by running: " + pico.cyan(updateMessage) + "\n"
        )
      );
    }
    process.exit();
  } catch {
  }
}

// src/generated/imports.ts
var templates = {
  "next": {
    "cli-abi": () => import("./_meta-OJJF5R46.js"),
    "cli-erc": () => import("./_meta-WMC6ITIM.js"),
    "cli-etherscan": () => import("./_meta-RXWBWOFU.js"),
    "cli-foundry": () => import("./_meta-G2NY2NZT.js"),
    "connectkit": () => import("./_meta-W5LQXLNS.js"),
    "default": () => import("./_meta-SAASD3AY.js"),
    "rainbowkit": () => import("./_meta-BFZPSEJL.js"),
    "web3modal": () => import("./_meta-7CNEHYDZ.js")
  },
  "vite-react": {
    "cli-abi": () => import("./_meta-7JMZB3LZ.js"),
    "cli-erc": () => import("./_meta-RWCM4RCO.js"),
    "cli-etherscan": () => import("./_meta-VGTJAWLV.js"),
    "cli-foundry": () => import("./_meta-X5SLRJQ3.js"),
    "connectkit": () => import("./_meta-EECVV4GW.js"),
    "default": () => import("./_meta-MZTLOE4I.js"),
    "rainbowkit": () => import("./_meta-EQZ6NJWD.js"),
    "web3modal": () => import("./_meta-JAZII2VU.js")
  }
};
var templateFrameworks = {
  "next": () => import("./_meta-DFXBVDK3.js"),
  "vite-react": () => import("./_meta-FQEQNN56.js")
};

// src/utils/templates.ts
var getTemplatesByFramework = async () => {
  var _a;
  const templateMeta = {};
  for (const [frameworkName, templateImports] of Object.entries(templates)) {
    for (const [templateName, templateImport] of Object.entries(
      templateImports
    )) {
      const template = (await templateImport()).default;
      const newTemplate = {
        ...template,
        name: templateName,
        framework: frameworkName,
        id: `${frameworkName}${template.default ? "" : `-${templateName}`}`
      };
      if (!templateMeta[frameworkName])
        templateMeta[frameworkName] = [newTemplate];
      else
        (_a = templateMeta[frameworkName]) == null ? void 0 : _a.push(newTemplate);
    }
  }
  return templateMeta;
};
var getTemplates = async () => {
  return Object.values(await getTemplatesByFramework()).flat();
};
var getTemplateFramework = ({
  templateId,
  templatesByFramework
}) => {
  for (const [frameworkName, templates2] of Object.entries(
    templatesByFramework
  )) {
    if (templates2.find((template) => template.id === templateId))
      return frameworkName;
  }
};
var getTemplateFrameworks = async () => Promise.all(
  Object.entries(templateFrameworks).map(async ([name, import_]) => ({
    ...(await import_()).default,
    id: name,
    name
  }))
);

// src/utils/validate.ts
import fs from "fs-extra";
import pico2 from "picocolors";
import validatePackageName from "validate-npm-package-name";
import path from "path";
async function validateProjectName({
  projectName,
  projectPath
}) {
  const nameValidation = validatePackageName(projectName);
  if (!nameValidation.validForNewPackages) {
    const problems = [
      ...nameValidation.errors ?? [],
      ...nameValidation.warnings ?? []
    ];
    return {
      valid: false,
      message: `\u{1F648} "${projectName}" is not a valid project name.`,
      problems: problems.map((problem) => `\u{1F449} ${problem}`).join("\n")
    };
  }
  const targetPath = path.join(process.cwd(), projectPath);
  if (await fs.pathExists(targetPath))
    return {
      valid: false,
      message: `\u{1F648} the directory "${projectPath}" already exists.`,
      problems: "\u{1F449} choose another name or delete the directory."
    };
  return {
    valid: true
  };
}
async function validateTemplateName({
  isNameRequired = true,
  templateId,
  templates: templates2
}) {
  if (isNameRequired && !templateId)
    return {
      valid: false,
      message: `\u{1F648} no template provided.`,
      problems: "\u{1F449} select a template or provide one using --template."
    };
  if (templateId && !templates2.find((template) => template.id === templateId))
    return {
      valid: false,
      message: `\u{1F648} the template "${templateId}" does not exist.`,
      problems: `Choose a valid name. Available: ${templates2.map(({ id }) => id).join(", ")}`
    };
  return { valid: true };
}
var ValidationError = class extends Error {
  name = "ValidationError";
  constructor(validation) {
    super([pico2.red(validation.message), validation.problems].join("\n"));
  }
};

// src/utils/injectProviders.ts
import fs2 from "fs-extra";

// src/meta.ts
var providers = {
  alchemy: {
    name: "alchemy",
    title: "Alchemy",
    description: "Learn more: https://wagmi.sh/react/providers/alchemy",
    apiKey: {
      env: "ALCHEMY_API_KEY",
      url: "https://dashboard.alchemyapi.io/"
    }
  },
  infura: {
    name: "infura",
    title: "Infura",
    description: "Learn more: https://wagmi.sh/react/providers/infura",
    apiKey: {
      env: "INFURA_API_KEY",
      url: "https://infura.io/login"
    }
  },
  public: {
    name: "public",
    title: "Public RPC",
    description: "Learn more: https://wagmi.sh/react/providers/public",
    apiKey: null
  }
};

// src/utils/injectProviders.ts
import path2 from "path";
function injectProviders({
  envNamespace,
  envPrefix,
  envVars,
  targetPath,
  providerNames
}) {
  if (Object.values(envVars).length > 0) {
    if (!fs2.existsSync(path2.join(targetPath, ".env")))
      fs2.writeFileSync(path2.join(targetPath, ".env"), "");
    if (!fs2.existsSync(path2.join(targetPath, ".env.example")))
      fs2.writeFileSync(path2.join(targetPath, ".env.example"), "");
    Object.entries(envVars).forEach(([key, value]) => {
      fs2.appendFileSync(
        path2.join(targetPath, ".env"),
        `${envPrefix}${key}=${value}
`
      );
      fs2.appendFileSync(
        path2.join(targetPath, ".env.example"),
        `${envPrefix}${key}=
`
      );
    });
  }
  const configPath = path2.join(targetPath, "src", "wagmi.ts");
  let src = fs2.readFileSync(configPath).toString();
  src = src.replace(
    /import { publicProvider } from 'wagmi\/providers\/public'/,
    providerNames.map(
      (providerName) => `import { ${providerName}Provider } from 'wagmi/providers/${providerName}'`
    ).join("\n")
  );
  src = src.replace(
    /publicProvider\(\)/,
    providerNames.map((providerName) => {
      const provider = providers[providerName];
      return `
    ${provider.name}Provider(${provider.apiKey ? `{ apiKey: ${envNamespace}.${envPrefix}${provider.apiKey.env}! }` : ""}),`;
    }).join("") + "\n  "
  );
  fs2.writeFileSync(configPath, src);
}

// src/utils/selectProviders.ts
import pico3 from "picocolors";
import { default as prompts } from "prompts";
async function selectProviders() {
  const envVars = {};
  let providerNames = (await prompts({
    name: "providerNames",
    message: "What providers would you like to use?",
    type: "multiselect",
    instructions: pico3.blue(
      pico3.italic(
        `
${pico3.bold("space")} to select, ${pico3.bold(
          "enter"
        )} to submit/skip`
      )
    ),
    choices: Object.entries(providers).map(([name, provider]) => ({
      ...provider,
      value: name
    }))
  })).providerNames;
  if (providerNames.length === 0)
    providerNames = ["public"];
  for (const providerName of providerNames) {
    const errorMessage = "API Key is required.";
    const provider = providers[providerName];
    if (!provider)
      continue;
    if (!provider.apiKey)
      continue;
    const apiKey = (await prompts({
      name: "apiKey",
      message: `What is your ${provider.title} API Key?
${pico3.blue(
        `Find it at: ${provider.apiKey.url}`
      )}
`,
      type: "text",
      validate(projectId) {
        if (!projectId)
          return errorMessage;
        return true;
      }
    })).apiKey;
    if (!apiKey)
      throw new Error(errorMessage);
    envVars[provider.apiKey.env] = apiKey;
  }
  return { envVars, providerNames };
}

// src/utils/warnings.ts
import pico4 from "picocolors";
var log2 = console.log;
function rpcProviderWarning({
  readMoreUrl = "https://wagmi.sh/docs/getting-started#configure-chains"
} = {}) {
  log2(
    `${pico4.yellow(
      "Important note:"
    )} It is HIGHLY recommended that you add an ${pico4.bold(
      pico4.underline(`alchemyProvider`)
    )}, ${pico4.bold(pico4.underline("infuraProvider"))}, or alike to ${pico4.blue(
      "src/wagmi.ts"
    )} before deploying your project to production to prevent being rate-limited.`
  );
  log2();
  log2(`Read more: ${pico4.blue(pico4.underline(readMoreUrl))}`);
}

export {
  package_default,
  createTemplate,
  getPackageManager,
  providers,
  injectProviders,
  notifyUpdate,
  selectProviders,
  getTemplatesByFramework,
  getTemplates,
  getTemplateFramework,
  getTemplateFrameworks,
  validateProjectName,
  validateTemplateName,
  ValidationError,
  rpcProviderWarning
};
