#!/usr/bin/env node
import {
  ValidationError,
  getPackageManager,
  getTemplateFramework,
  getTemplateFrameworks,
  getTemplates,
  getTemplatesByFramework,
  notifyUpdate,
  package_default,
  validateProjectName,
  validateTemplateName
} from "./chunk-3UIWA75U.js";

// src/index.ts
import { cac } from "cac";
import cpy from "cpy";
import { execa } from "execa";
import fs from "fs-extra";
import { oraPromise } from "ora";
import pico from "picocolors";
import { default as prompts } from "prompts";
import path from "path";
import { fileURLToPath } from "url";
var log = console.log;
async function run({
  args,
  options,
  templates
}) {
  var _a, _b, _c, _d, _e, _f, _g;
  if (options.help)
    return;
  log();
  log(
    `Welcome to ${pico.bold(
      pico.blue("create-wagmi")
    )} \u2013 the quickest way to get started with wagmi!`
  );
  log();
  const __dirname = fileURLToPath(new URL(".", import.meta.url));
  const templatesPath = path.join(__dirname, "..", "templates");
  let templateId = options.template || options.t;
  let templateValidation = await validateTemplateName({
    isNameRequired: false,
    templateId,
    templates
  });
  if (!templateValidation.valid)
    throw new ValidationError(templateValidation);
  let projectName;
  let projectPath;
  if (args[0]) {
    projectPath = args[0].trim();
    const splitPath = projectPath.split("/");
    projectName = ((_a = splitPath[splitPath.length - 1]) == null ? void 0 : _a.trim()) || "";
    log(pico.green("\u2714"), pico.bold(`Using project name:`), projectName);
  } else {
    const res = await prompts({
      initial: "my-app",
      name: "projectName",
      message: "What is your project named?",
      type: "text",
      async validate(projectName2) {
        const validation = await validateProjectName({
          projectName: projectName2,
          projectPath: projectName2
        });
        if (!validation.valid)
          return validation.message;
        return true;
      }
    });
    projectName = (_b = res.projectName) == null ? void 0 : _b.trim();
    projectPath = projectName;
  }
  const nameValidation = await validateProjectName({
    projectName,
    projectPath
  });
  if (!nameValidation.valid)
    throw new ValidationError(nameValidation);
  const frameworks = await getTemplateFrameworks();
  const templatesByFramework = await getTemplatesByFramework();
  let frameworkId = getTemplateFramework({
    templateId,
    templatesByFramework
  }) || ((_c = frameworks[0]) == null ? void 0 : _c.id);
  if (!templateId || !frameworkId) {
    frameworkId = (await prompts({
      name: "frameworkId",
      message: "What framework would you like to use?",
      type: "select",
      choices: frameworks.map(({ name, ...t }) => ({
        ...t,
        value: name
      })).sort((a, b) => a.default && !b.default ? -1 : 1)
    })).frameworkId;
    if (!frameworkId)
      return;
    const frameworkTemplates = templatesByFramework[frameworkId];
    if (!frameworkTemplates)
      return;
    templateId = (await prompts({
      name: "templateId",
      message: "What template would you like to use?",
      type: "select",
      choices: frameworkTemplates.map(({ id, ...t }) => ({
        ...t,
        value: id
      })).sort((a, b) => a.default && !b.default ? -1 : 1)
    })).templateId;
  }
  const frameworkMeta = frameworks.find(({ id }) => id === frameworkId);
  if (!frameworkMeta)
    return;
  const templateMeta = templates.find(({ id }) => id === templateId);
  if (!templateMeta)
    return;
  const context = (() => {
    let ctx = {};
    const get = () => ctx;
    const set = (ctx_) => ctx = { ...ctx, ...ctx_ };
    return { get, set };
  })();
  const hooks = templateMeta.hooks;
  await ((_d = hooks == null ? void 0 : hooks.afterValidate) == null ? void 0 : _d.call(hooks, { context }));
  templateValidation = await validateTemplateName({
    templateId,
    templates
  });
  if (!templateValidation.valid)
    throw new ValidationError(templateValidation);
  const targetPath = path.join(process.cwd(), projectPath);
  log(`Creating a new wagmi app in ${pico.green(targetPath)}.`);
  log();
  log(
    `Using ${pico.bold(frameworkMeta.title)}${!templateMeta.default ? ` with ${pico.bold(templateMeta.title)}` : ""}.`
  );
  log();
  const templatePath = path.join(
    templatesPath,
    frameworkMeta.name,
    templateMeta.name
  );
  await cpy(path.join(templatePath, "**", "*"), targetPath, {
    filter: (file) => file.name !== "_meta.ts",
    rename: (name) => name.replace(/^_dot_/, ".")
  });
  const packageJson = await fs.readJSON(path.join(targetPath, "package.json"));
  packageJson.name = projectName;
  await fs.writeFile(
    path.join(targetPath, "package.json"),
    JSON.stringify(packageJson, null, 2)
  );
  const packageManager = await getPackageManager({ options });
  if (packageManager === "npm") {
    await fs.appendFile(
      path.join(targetPath, ".npmrc"),
      "\nlegacy-peer-deps = true"
    );
  }
  await ((_e = hooks == null ? void 0 : hooks.beforeInstall) == null ? void 0 : _e.call(hooks, { context, targetPath }));
  log(`Using ${pico.bold(packageManager)}.`);
  log();
  const installArgs = [
    "install",
    packageManager === "npm" ? "--quiet" : "--silent"
  ];
  await oraPromise(
    execa(packageManager, installArgs, {
      cwd: targetPath,
      env: {
        ...process.env,
        ADBLOCK: "1",
        DISABLE_OPENCOLLECTIVE: "1",
        NODE_ENV: "development"
      }
    }),
    {
      text: "Installing packages. This may take a couple of minutes.",
      failText: "Failed to install packages.",
      successText: "Installed packages."
    }
  );
  log();
  await ((_f = hooks == null ? void 0 : hooks.afterInstall) == null ? void 0 : _f.call(hooks, { context, targetPath }));
  if (!options.skipGit) {
    await execa("git", ["init"], { cwd: targetPath });
    await execa("git", ["add", "."], { cwd: targetPath });
    await execa(
      "git",
      [
        "commit",
        "--no-verify",
        "--message",
        "Initial commit from create-wagmi"
      ],
      { cwd: targetPath }
    );
    log(pico.green("\u2714"), "Initialized git repository.");
    log();
  }
  log("\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015");
  log();
  log(
    `${pico.green("Success!")} Created ${pico.bold(
      projectName
    )} at ${pico.green(path.resolve(projectPath))}`
  );
  log();
  log(
    `To start your app, run \`${pico.bold(
      pico.cyan(`cd ${projectPath}`)
    )}\` and then \`${pico.bold(
      pico.cyan(
        `${packageManager}${packageManager === "npm" ? " run" : ""} dev`
      )
    )}\``
  );
  log();
  log("\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015\u2015");
  log();
  await ((_g = hooks == null ? void 0 : hooks.afterSetup) == null ? void 0 : _g.call(hooks, { context, targetPath }));
}
(async () => {
  let templates;
  try {
    templates = await getTemplates();
  } catch (err) {
    log(pico.red(err.message));
    process.exit(1);
  }
  const cli = cac(package_default.name).version(package_default.version).usage(`${pico.green("<project-directory>")} [options]`).option(
    "-t, --template [name]",
    `A template to bootstrap with. Available: ${templates.sort((a, b) => a.default && !b.default ? -1 : 1).map(({ id }) => id).join(", ")}`
  ).option("--npm", "Use npm as your package manager").option("--pnpm", "Use pnpm as your package manager").option("--yarn", "Use yarn as your package manager").option("--skip-git", "Skips initializing the project as a git repository").help();
  const { args, options } = cli.parse(process.argv);
  try {
    await run({ args, options, templates });
    log();
    await notifyUpdate({ options });
  } catch (error) {
    log(
      error instanceof ValidationError ? error.message : pico.red(error.message)
    );
    log();
    await notifyUpdate({ options });
    process.exit(1);
  }
})();
